/*
 * Created on 1-Oct-2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.blazinggames.util.conset;

/**
 * <one line to give the program's name and a brief idea of what it does.>
 * Copyright (C) 2003 Billy D. Spelchan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 0.00
 */
public class Tilemap 
{
// *********************
// ***** VARIABLES *****
// *********************

	/** holds the actual level data (tile size is int) */
	protected int _levelData[];
	/** width of a row of tiles */
	protected int _width;
	/** how many rows of tiles in a level */
	protected int _height;
	/** if data not wrapped, value to return if requested tile is out of bounds (default 0) */
	protected int _outOfBoundsTile;
	/** if true, out of bound tile requests will be wrapped into valid tile boundry (default false) */
	protected boolean _wrapData;
	

// ************************
// ***** CONSTRUCTORS *****
// ************************

	public Tilemap()
	{
		this(1, 1);
	}
	
	public Tilemap(int w, int h)
	{
		_levelData = new int[w*h];
		_width = w;
		_height = h;
		_outOfBoundsTile = 0;
		_wrapData = false;
		clear(0);
	}


// ***************************************************
// ***** Methods: Getting, setting, and resizing *****
// ***************************************************

	/**
	 * Gets tile integer value. If wrapping is enabled, automatically will wrap coordinates.
	 * If wrapping not enabled, out of bounds requests will return the designated out of bounds tile.
	 * 
	 * @param x Desired x location. Should be positive value.
	 * @param y Desired y location. Should be positive value.
	 * @return tile at location (wrapped if wrapping enabled). 
	 */
	public synchronized int getTile(int x, int y)
	{
		int offset, result;
		if ( (x >=_width) || (y >= _height) )
		{
			if (_wrapData)
			{
				offset = (y % _height) * _width + (x % _width);
				result = _levelData[offset];
			}
			else
				result = _outOfBoundsTile;
		}
		else
		{
			offset = y * _width + x;
			result = _levelData[offset];
		} 

		return result;
	}
	
	/**
	 * Sets tile integer value. If wrapping is enabled, automatically will wrap coordinates.
	 * If wrapping not enabled, out of bounds requests will be ignored.
	 * 
	 * @param x Desired x location. Should be positive value.
	 * @param y Desired y location. Should be positive value. 
	 */
	public synchronized void setTile(int x, int y, int value)
	{
		int offset;
		if ( (x >=_width) || (y >= _height) )
		{
			if (_wrapData)
			{
				offset = (y % _height) * _width + (x % _width);
				_levelData[offset] = value;
			}
		}
		else
		{
			offset = y * _width + x;
			_levelData[offset] = value;
		} 		
	}
	
	/**
	 * Returns the state of map wrapping. 
	 * Wrapping is when out of bounds values wrap around the map as if map was a globe.
	 * @return true if map wraps, otherwise false.
	 */
	public boolean getWrap()
	{
		return _wrapData;
	}
	
	/**
	 * Sets the state of map wrapping. 
	 * Wrapping is when out of bounds values wrap around the map as if map was a globe.
	 */
	public void setWrap(boolean b)
	{
		_wrapData = b;
	}
	
	/**
	 * If map wrapping is not enabled, requests for out of bound tiles will return
	 * the current out of bounds value, which is also what this returns.
	 * 
	 * @return current value assigned to out of bounds tiles.
	 */
	public int getOutOfBoundsTile()
	{
		return _outOfBoundsTile;
	}
	
	/**
	 * If map wrapping is not enabled, requests for out of bound tiles will return
	 * the current out of bounds value, which is what you are assigning here.
	 * 
	 * @param value the out of bounds tile value (default is 0).
	 */
	public void setOutOfBoundsTile(int value)
	{
		_outOfBoundsTile = value;
	}
	
	/**
	 * Resizes the tile data to the indicated size.
	 * If larger than current size then extra space on right and bottom will be created.
	 * This empty space will be filled with the out of bounds tile.
	 * If same size as current size then nothing happens.
	 * If smaller than current size then right and bottom are clipped.
	 * 
	 * @param w new width of map. Warning: Must be > 1.
	 * @param h new height of map. Warning: Must be > 1.
	 */
	public synchronized void resize(int w, int h)
	{
		if ( (w == _width) && (h == _height) )
			return;
		int newData[] = new int[w * h];
		boolean currentWrap = _wrapData;
		_wrapData = false;
		int offset = 0;
		for(int cntrY = 0; cntrY < h; ++cntrY)
		{
			for (int cntrX = 0; cntrX < w; ++cntrX)
			{
				newData[offset] = getTile(cntrX, cntrY);
				++offset;
			}
		}
		_wrapData = currentWrap;
		_levelData = newData;
		_width = w;
		_height = h;
	}
	
	/**
	 * returns the width of the tile map.
	 * @return the width of the tile map.
	 */
	public int getWidth()
	{
		return _width;
	}
	
	/**
	 * returns the height of the tile map.
	 * @return the height of the tile map.
	 */
	public int getHeight()
	{
		return _height;
	}


// ******************************************************************	
// ***** Methods: Editing Aids - clearing, copying, and pasting *****
// ******************************************************************

	/**
	 * @param n value to fill map with.
	 */
	public void clear(int n)
	{
		for (int cntr = 0; cntr < _levelData.length; ++cntr)
			_levelData[cntr] = n;
	}
	
	/**
	 * Fills the indicated region with the indicated value.
	 * 
	 * @param x starting x location
	 * @param y starting y location
	 * @param w width of region to clear
	 * @param h height of region to clear
	 * @param value value to fill region with
	 */
	public void	clearRegion(int x, int y, int w, int h, int value)
	{
		for (int cntrX = 0; cntrX < w; ++cntrX)
			for (int cntrY = 0; cntrY < h; ++cntrY)
				setTile(x + cntrX, y + cntrY, value);
	}
	
	/**
	 * Makes a Tilemap that contains the data from the indicated region.
	 * If region larger than map, results will be based on wrap data and out of bounds tile settings.
	 * 
	 * @param x x location to start creation of tilemap from
	 * @param y y location to start creation of tilemap from
	 * @param w width of the tilemap
	 * @param h height of the tilemap
	 * @return new tilemap containing copy of indicated region
	 */
	public Tilemap copyRegion(int x, int y, int w, int h)
	{
		Tilemap result = new Tilemap(w, h);
		for (int cntrX = 0; cntrX < w; ++cntrX)
			for(int cntrY = 0; cntrY < h; ++cntrY)
				result.setTile(cntrX, cntrY, getTile(x + cntrX, y + cntrY));
		return result;
	}
	
	/**
	 * Pastes tilemap into indicated location.
	 * If region larger than map, results will be based on wrap data and out of bounds tile settings.
	 * 
	 * @param paste data to be pasted
	 * @param x starting x location for pasting
	 * @param y starting y location for pasting
	 */
	public void pasteRegion(Tilemap paste, int x, int y)
	{
		int w = paste.getWidth();
		int h = paste.getHeight();
		for (int cntrX = 0; cntrX < w; ++cntrX)
			for(int cntrY = 0; cntrY < h; ++cntrY)
				setTile(x + cntrX, y + cntrY, paste.getTile(cntrX, cntrY));
	}
	
}
